% Script TimingBasis
% synthesise timed response using timing basis (Yamazaki & Tanaka 2005 Neural Comp)

%%
% set up time base
tmax = 1; nt = 1001;
t = linspace(0, tmax, nt)';

%% set up 'chaotic timing basis' to be triggered on parallel fibres

% Recurrent network params
nw = 100;    % number of neurons: in paper nw = 1000
tau = 100;   % neuron TC
kappa = 1;   % related to total inhibition on neuron
gamma = 1/tau;

% set seed for repeatability
rand('seed',  2);

% random inhibitory weights: Yamazaki et al. prescription
R = rand(nw, nw);
R(R <= 0.5) = 0;
R(R > 0.5)  = 2*kappa/nw;

% common constant input
I = ones(nt, 1);

% storage
u = zeros(nt, nw); % leaky integrator accumulator
p = zeros(nt, nw); % firing rate

p(1, :) = I(1);
for i = 2:nt
    % update leaky integrator
    u(i, :) = (1-gamma)*u(i-1, :)+p(i-1, :);

    % recurrent conwectivity
    p(i, :) = I(i)-u(i, :)*R;

    % take positive part
    p(i, p(i, :) < 0) = 0;
end

% this kinda replicates their Fig 1 (left panel)
fig(1, 6, 4); clf
plot(t, p(:, 1:50))
axis([0 tmax 0 1]);
title('Recurrent network: 50 sample PF outputs');
xlabel('Time after trigger (sec)');
ylabel('Neuron responses')

%% set up desired response
% Gaussian pause inspired by Classical Conditioning)
m = 0.5; s = 0.1; t = linspace(0, 1, nt)';
zd = 1-0.9*exp(-0.5*((t-m)/s).^2);

% plot desired response
fig(2, 6, 4); clf
fig(2, 6, 4); clf
plot(t, zd);
axis([0 tmax -0.1 1.1])
title('Desired response')
xlabel('time')

%% Batch learning of PF/PC weights

w = zeros(nw, 1); % weights initially zero
beta = 0.1/nt;   % learning rate (normalised)
nbatch = 1000;    % number of presentations of batch
e = NaN(nt, 1);   % storage for teaching signal

for j = 1:nbatch
    % output of Purkinje cell
    z = 1+p*w; % PF input modulates tonic rate of 1

    % output error
    e = z-zd;

    % covariance learning
    w = w-beta*p'*e;
    
    % watch learning progress: comment out for efficiency
    fig(3, 6, 4); clf
    plot(t, zd, t, z)
    axis([0 tmax -0.1 1.1])
    title(sprintf('Batch %d of %d', j, nbatch))
    drawnow
end

%% Compare with optimal weights using batch least squares
wopt = p\zd;
zopt = p*wopt;

fig(4, 6, 4); clf
plot(t, zd, t, zopt, t, z)
title('Compare optimal and learned reconstruction')
xlabel('time')
legend('desired', 'optimal', 'learned', 'Location', 'best')

% figure(1);
% print -djpeg -r300 Fig4c
% 
% figure(2);
% print -djpeg -r300 Fig4d


